#client.AdriftGameScreen.py

import sys
import os
import pygame
from pygame.locals import *

import gui
import defaultStyle

from engine.Gameworld import Gameworld

from engine.AtlanSpriteGroup import AtlanSpriteGroup

from common.Utility import *

from Goal import Goal

from Boat import Boat

from Cloud import Cloud

from Block import Block

from PatrolBoat import PatrolBoat

from Canon import Canon

from Pirate import Pirate

from GunPirate import GunPirate

from DeathScreen import DeathScreen

from Fireworks import Fireworks

from GunPatrolBoat import GunPatrolBoat

from Doubloon import Doubloon

from Seagull import Seagull

import random

import math

GAME = 0
PAUSED = 1
VICTORY = 2
DEFEAT = 3

class AdriftGameScreen(Gameworld):
	"""
		Main game screen.
		Handles most of the logic for when the user is actually
		controlling a boat and trying to reach the goals.
	"""

	def __init__(self):
		"""
			Create the game screen.
		"""
		#inner and outer bounds of the camera
		#used to check if the screen needs to scroll
		self.outer = Rect(0, 0, 800, 600)
		self.inner = Rect(200, 200, 400, 200)
		self.outerCamX = 0.0
		self.outerCamY = 0.0
		self.innerCamX = 100.0
		self.innerCamY = 100.0
		#the boat and obstacles
		self.sprites = AtlanSpriteGroup()
		self.clouds = AtlanSpriteGroup()
		self.birds = AtlanSpriteGroup()
		
		defaultStyle.init(gui)
		
		#create the GUI
		self.app = gui.Desktop()
		
		self.labelStyleCopy = gui.defaultLabelStyle.copy()
		self.labelStyleCopy['border-width'] = 0
		self.labelStyleCopy['wordwrap'] = True
		self.labelStyleCopy['autosize'] = False
		
		#blue background image
		self.backgroundImage = getImage("oceanBackground")
		
		#the background takes up the whole screen
		self.backgroundBounds = Rect(0, 0, 800, 600)
		
		#image for the user to see if its time to blow again or not
		self.okToBlowImage = getImage("okSign")
		self.okSignBounds = Rect(745, 0, 50, 50)

		
		self.clickPoint = None
		self.releasePoint = None
		self.windStrength = 0
		self.windAngle = 0
		self.goal = None
		self.boat = None
		self.mode = GAME
		self.startButton = None
		
		self.defeatTimer = 0
		
		self.drawArrowTimer = 0
		self.drawArrowTimerMax = 5000
		self.drawArrow = False
		
		self.drawLine = False
		
		self.currentMousePosition = None
		
		self.name = "ERROR NO VALID NAME"
		
		self.font = getGameFont()
		self.currentTime = 0
		self.bestTime = 0
		

		
		self.worldWidth = 1600
		self.worldHeight = 1600
		self.screenWidth = 800
		self.screenHeight = 600
		
		self.screen = pygame.Rect(0, 0, self.screenWidth, self.screenHeight)
		
		self.deathScreen = None
		self.fireworks = None
		self.foundDoubloon = False
		
		self.waveSound = None
		self.music = None
		self.applauseSound = None
		
	def inView(self):
		"""
			Return all sprites that are visible to the player.
		"""
		viewable = []
		# check each sprite in the entities of "physics"
		for sprite in self.sprites:
			# if the collision occurs with the camera
			#b = sprite.getImageBounds()
			self.outer = pygame.Rect(int(self.outerCamX), int(self.outerCamY), self.outer.width, self.outer.height)
			if sprite.bounds.colliderect(self.outer):
				# tell the sprite to think of where it is in relative terms of the camera
				sprite.calculateCameraCoords(self.outer)
				# add the sprite to the list of things to draw
				viewable.append(sprite)
		return viewable
		
	def inViewBirds(self):
		"""
			Return all the birds that are visible to the player.
		"""
		viewable = []
		# check each sprite in the entities of "physics"
		for sprite in self.birds:
			# if the collision occurs with the camera
			#b = sprite.getImageBounds()
			self.outer = pygame.Rect(int(self.outerCamX), int(self.outerCamY), self.outer.width, self.outer.height)
			if sprite.bounds.colliderect(self.outer):
				# tell the sprite to think of where it is in relative terms of the camera
				sprite.calculateCameraCoords(self.outer)
				# add the sprite to the list of things to draw
				viewable.append(sprite)
		return viewable
		
	def inViewClouds(self):
		"""
			Return all the clouds that are visible to the player.
		"""
		viewable = []
		# check each sprite in the entities of "physics"
		for sprite in self.clouds:
			# if the collision occurs with the camera
			#b = sprite.getImageBounds()
			self.outer = pygame.Rect(int(self.outerCamX), int(self.outerCamY), self.outer.width, self.outer.height)
			if sprite.bounds.colliderect(self.outer):
				# tell the sprite to think of where it is in relative terms of the camera
				sprite.calculateCameraCoords(self.outer)
				# add the sprite to the list of things to draw
				viewable.append(sprite)
		return viewable
		
	def handleEvent(self, event):
		"""
			Handles any events generated by the user.
		"""
		#quit when the user clicks the "X"
		if event.type == pygame.QUIT:
			sys.exit()
		elif event.type == pygame.MOUSEMOTION:
			self.handleMouseMoved(event.pos)
		elif event.type == pygame.MOUSEBUTTONDOWN:
			if event.button == 1:
				self.handleMouseDown(event.pos)
		elif event.type == pygame.MOUSEBUTTONUP:
			if event.button == 1:
				self.handleMouseUp(event.pos)
				
	def scroll(self, rect, speed):
		"""
			Scroll the screen so that it contains a rectangle.
			Scroll at a given speed to reduce jerkiness.
		"""
		# if the rect exceeds the inner boundry, take steps to correct this by adjusting the camera's position
		if not self.inner.contains(rect):
			if rect.x > self.inner.x + self.inner.width:
				#if self.outerCamX + speed < self.worldWidth:
				self.outerCamX += speed
				self.innerCamX += speed
			elif rect.x < self.inner.x:		
				#if self.outerCamX - speed > 0:				
				self.outerCamX -= speed
				self.innerCamX -= speed
			if rect.y > self.inner.y + self.inner.height:
				self.outerCamY += speed
				self.innerCamY += speed
			elif rect.y < self.inner.y:
				self.outerCamY -= speed
				self.innerCamY -= speed
			self.inner = pygame.Rect(int(self.innerCamX), int(self.innerCamY), self.inner.width, self.inner.height)
		
			
	def tick(self, time):
		"""
			Update the gui.
		"""
		if self.drawArrow:
			self.drawArrowTimer += time
			if self.drawArrowTimer >= self.drawArrowTimerMax:
				self.drawArrow = False
				self.drawArrowTimer = 0
		if self.mode == GAME:
			#update game logic
			self.currentTime += time
			for sprite in self.sprites:
				sprite.setWind(self.windStrength, self.windAngle)
				sprite.update(time)
				sprite.checkCollision(self.sprites)
				if sprite.dead:
					self.sprites.remove(sprite)
			for sprite in self.clouds:
				sprite.setWind(self.windStrength * 2, self.windAngle)
				sprite.update(time)
		elif self.mode == DEFEAT:
			#update lose screen logic
			self.deathScreen.update(time)
			self.defeatTimer += time
			if self.defeatTimer >= 4000:
				self.controller.getScene(1).restartLevel()
				self.controller.changeScene(1)
				self.defeatTimer = 0
		elif self.mode == VICTORY:
			#update victory screen logic
			self.fireworks.update(time)
		for bird in self.birds:
			#update the birds
			bird.update(time)
		#update the gui
		self.app.update()
		
		
	def acquiredDoubloon(self):
		"""
			Keep track of the player having found the gold in this map.
		"""
		self.foundDoubloon = True
		
	def victory(self):
		"""
			The player reached the goal.
			Stop the music, start the victory music.
			Create a button that leads back to the level select screen.
			Start drawing fireworks.
		"""
		self.waveSound.stop()
		self.music.stop()
		self.applauseSound = pygame.mixer.Sound(load_sound("applause.ogg"))
		self.applauseSound.play(loops=-1)
		self.mode = VICTORY
		self.startButton = gui.Button(position = (300, 500), size = (200,0), parent = self.app, text = "NEXT LEVEL")
		self.startButton.onClick = self.handleStart
		self.fireworks = Fireworks(self)
		
	def handleStart(self, button):
		"""
			The player has chosen to stop watching the victory screen
			and head back to the select screen.
			Stop the victory sound, change back to select screen.
		"""
		self.applauseSound.stop()
		self.controller.getScene(1).advance(self.name, self.currentTime, self.foundDoubloon)
		self.controller.changeScene(1)
		
	def reset(self):
		"""
			Return the game to the default state for playing a new level.
		"""
		self.foundDoubloon = False
		self.deathScreen = None
		self.fireworks = None
		self.outerCamX = 0.0
		self.outerCamY = 0.0
		self.innerCamX = 100.0
		self.innerCamY = 100.0
		if self.startButton != None:
			self.app.remove(self.startButton)
		self.mode = GAME
		self.boat = None
		self.sprites.empty()
		self.clouds.empty()
		self.clickPoint = None
		self.releasePoint = None
		self.windStrength = 0
		self.windAngle = 0
		self.goal = None
		self.startButton = None
		self.drawArrow = False
		self.drawLine = False
		self.currentMousePosition = None
		self.drawArrowTimer = 0
		self.currentTime = 0
		
	def sunk(self):
		"""
			The player collided with an obstacle.
			Make an alarm sound, draw the lose screen.
		"""
		if self.mode != VICTORY:
			self.waveSound.stop()
			self.music.stop()
			s = pygame.mixer.Sound(load_sound("alarm.ogg"))
			s.play()
			self.mode = DEFEAT
			self.deathScreen = DeathScreen(self)
		
	def draw(self, surface):
		"""
			Draw the game.
			Has a bunch of layers to do drawing:
			background, sprites, birds, clouds, gui, arrow, text
		"""
		#draw the background image
		surface.blit(self.backgroundImage, self.backgroundBounds)
		
		boundsX = 0 - self.outer.x
		boundsY = 0 - self.outer.y
		#draw boundary for the world
		pygame.draw.rect(surface, (255, 255, 255), pygame.Rect(boundsX, boundsY, self.worldWidth, self.worldHeight), 3)
		
		#draw sprites
		for sprite in self.inView():
			sprite.draw(surface)
			
		#draw birds
		visible = self.inViewBirds()
		for sprite in visible:
			sprite.draw(surface)
			
		#draw clouds
		visible = self.inViewClouds()
		for sprite in visible:
			sprite.draw(surface)

		#draw gui
		self.app.draw()
		#draw line to indicate wind direction
		if self.drawLine:
			pygame.draw.line(surface, (0, 0, 0), self.clickPoint, self.currentMousePosition, 3)
		#draw ok sign if it is ok to blow
		if not self.drawArrow:
			surface.blit(self.okToBlowImage, self.okSignBounds)
		
		#draw the name of the level
		surf = self.font.render(self.name, True, (0,0,0))
		surface.blit(surf, (50, 535, 100, 20)) 
		
		#draw the current time
		surf = self.font.render("Current Time:  " + str(self.currentTime), True, (0, 0, 0))
		surface.blit(surf, (50, 555, 100, 20)) 
		#draw the best time
		surf = self.font.render("BestTime:  " + str(self.bestTime), True, (0, 0, 0))
		surface.blit(surf, (50, 575, 100, 20)) 
		
		#draw death screen / victory screen if required
		if self.deathScreen != None:
			self.deathScreen.draw(surface)
			
		if self.fireworks != None:
			self.fireworks.draw(surface)
		
	def loadLevel(self, file):
		"""
			Parse a text file to figure out the stats for the level.
			Levels have a fixed format, any changes will cause an error.
		"""
		f = open("data" + os.sep + "levels" + os.sep + file)
		lineCounter = -1
		for line in f:
			lineCounter += 1
			if lineCounter == 0:
				#name is the first line of any file
				self.name = line.strip()
			elif lineCounter == 1:
				#best time is the second line of any file
				self.bestTime = int(line.strip())
			elif lineCounter == 2:
				#boat location is the third line of any file
				self.parseBoat(line.strip())
			elif lineCounter == 3:
				#number of clouds is the fourth line
				self.parseClouds(line.strip())
			elif lineCounter == 4:
				#goal location is the fifth line
				self.parseGoal(line.strip())
			elif lineCounter == 5:
				#doubloon location and whehter or not it has been found
				self.parseDoubloon(line.strip())
			elif lineCounter == 6:
				#what music file to play
				self.parseMusic(line.strip())
			else:
				#after the seventh line, the file contains
				#information about the obstacles
				self.parseObstacle(line.strip())
		f.close()
		#play background ocean noise
		self.music.play(loops=-1)
		self.waveSound = pygame.mixer.Sound(load_sound("ocean.ogg"))
		self.waveSound.play(loops=-1)
		#create 100 birds
		self.birds = AtlanSpriteGroup()
		for i in range(100):
			self.birds.add(Seagull(random.randint(0, 1600), random.randint(0, 1600)))
		
	def parseDoubloon(self, line):
		"""
			Figure out where to put the doubloon for this level.
			Also set whether or not the player has found it yet.
		"""
		parts = line.split(":")
		x = int(parts[0])
		y = int(parts[1])
		w = int(parts[2])
		h = int(parts[3])
		doubloonBounds = pygame.Rect(x, y, w, h)
		doubloon = Doubloon(doubloonBounds, self)
		self.sprites.add(doubloon)
		self.foundDoubloon = (parts[4] == 'True')
		
	def parseMusic(self, line):
		"""
			Set the music equal to what the file said it should be.
		"""
		self.music = pygame.mixer.Sound(load_sound(line))
		
	def parseBoat(self, line):
		"""
			Figure out where to place the player's boat.
			Also set the screen to center around the boat's location.
		"""
		parts = line.split(":")
		x = int(parts[0])
		y = int(parts[1])
		w = int(parts[2])
		h = int(parts[3])
		self.boat = Boat(Rect(x, y, w, h), self)
		self.boat.mode = self.boat.SCROLL
		self.sprites.add(self.boat)
		self.outer.centerx = self.boat.bounds.centerx
		self.outer.centery = self.boat.bounds.centery
		self.inner.centerx = self.boat.bounds.centerx
		self.inner.centery = self.boat.bounds.centery
		self.outerCamX = self.outer.x / 1.0
		self.outerCamY = self.outer.y / 1.0
		self.innerCamX = self.inner.x / 1.0
		self.innerCamY = self.inner.y / 1.0
		
	def parseClouds(self, line):
		"""
			Figure out how many clouds to put on the world.
		"""
		number = int(line.strip())
		for i in xrange(number):
			#i found that clustering them from 300 to 1600 absolute
			#coordinates tends to work best
			c = Cloud(Rect(random.randint(-300, 1600), random.randint(-300, 1600), 25, 25))
			c.mode = c.SCROLL
			self.clouds.add(c)
			
	def parseGoal(self, line):
		"""
			Figure out where to put the goal.
		"""
		parts = line.split(":")
		x = int(parts[0])
		y = int(parts[1])
		w = int(parts[2])
		h = int(parts[3])
		goalBounds = pygame.Rect(x, y, w, h)
		self.goal = Goal(goalBounds, self)
		self.sprites.add(self.goal)
		
	def parseObstacle(self, line):
		"""
			Figure out what kind of obstacle it is.
			Call the appropriate method to add it to the game world.
		"""
		if line.startswith("block"):
			self.parseBlock(line)
		elif line.startswith("patrol"):
			self.parsePatrolBoat(line)
		elif line.startswith("island"):
			self.parseIsland(line)
		elif line.startswith("canon"):
			self.parseCanon(line)
		elif line.startswith("pirate"):
			self.parsePirate(line)
		elif line.startswith("gunpirate"):
			self.parseGunPirate(line)
		elif line.startswith("gunpatrol"):
			self.parseGunPatrol(line)
			
	def parseBlock(self, line):
		"""
			Add a block to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		b = Block(pygame.Rect(x, y, w, h), self)
		self.sprites.add(b)
		
	def parsePatrolBoat(self, line):
		"""
			Add a patrol boat to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		speed = int(parts[5])
		#how far in pixels to travel
		distance = int(parts[6])
		hor = parts[7] == "horizontal"
		s = PatrolBoat(pygame.Rect(x, y, w, h), self, speed, distance, hor)
		self.sprites.add(s)
		
	def parseGunPatrol(self, line):
		"""
			Add a gun patrol boat to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		speed = int(parts[5])
		#how far in pixels to travel
		distance = int(parts[6])
		hor = parts[7] == "horizontal"
		s = GunPatrolBoat(pygame.Rect(x, y, w, h), self, speed, distance, hor)
		self.sprites.add(s)
		
	def parseCanon(self, line):
		"""
			Add a canon to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		i = Canon(pygame.Rect(x, y, w, h), self)
		self.sprites.add(i)
		
	def parsePirate(self, line):
		"""
			Add a pirate to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		i = Pirate(pygame.Rect(x, y, w, h), self)
		self.sprites.add(i)
		
	def parseGunPirate(self, line):
		"""
			Add a gun pirate to the game world.
		"""
		parts = line.split(":")
		x = int(parts[1])
		y = int(parts[2])
		w = int(parts[3])
		h = int(parts[4])
		i = GunPirate(pygame.Rect(x, y, w, h), self)
		self.sprites.add(i)
		
	def handleMouseMoved(self, pos):
		"""
			Update the current stored position for the mouse.
		"""
		self.currentMousePosition = pos
			
	def handleMouseDown(self, pos):
		"""
			If not currently drawing the wind, create the new 
			start point for the wind.
		"""
		if not self.drawArrow:
			self.clickPoint = pos
			self.drawLine = True
		
	def handleMouseUp(self, pos):
		"""
			The player released the mouse.
			Set the wind vector to the be line between where they
			clicked and where they released.
		"""
		if not self.drawArrow:
			self.releasePoint = pos
			self.setWind(self.clickPoint, self.releasePoint)
			self.drawLine = False
			self.drawArrow = True
		
	def setWind(self, start, end):
		"""
			Create wind in the direction from where they first clicked
			the mouse and where they let go.
			The strength of the wind is based on how far apart
			those two points are.
		"""
		distance = self.calculateDistance(start, end)
		strength = (distance / 800) * 3 + 1
		angle = self.calculateAngle(start, end)
		self.windStrength = strength
		self.windAngle = angle
		
	def calculateAngle(self, point1, point2):
		"""
			Calculate's the angle between a line connecting point 1 and point 2 and a line going through point 1 parallel to the x axis.
		"""
		inRads = math.atan2(point2[1] - point1[1], point2[0] - point1[0])
		return -1 * inRads * 57.295
		
	def calculateDistance(self, point1, point2):
		"""
			Calculate the absolute value of the distance between point 1 and point 2.
		"""
		return math.sqrt(((point1[0] - point2[0]) ** 2) +  ((point1[1] - point2[1]) ** 2))