#client.AdriftLoadScreen.py

import sys

from engine.Gameworld import Gameworld

import pygame
from pygame.locals import *

from common.Utility import *

import gui

import defaultStyle

class AdriftLoadScreen(Gameworld):
	"""
		The GUI screen that allows the player to pick which level to play.
		You can only play a stage if you have beaten the level that proceeds it (except for the first level).
	"""

	def __init__(self):
		"""
			Create all the buttons and such.
		"""
		defaultStyle.init(gui)
		
		#create the GUI
		self.app = gui.Desktop()
		
		self.labelStyleCopy = gui.defaultLabelStyle.copy()
		self.labelStyleCopy['border-width'] = 0
		self.labelStyleCopy['wordwrap'] = True
		self.labelStyleCopy['autosize'] = False
		
		self.levelDict = {}
		
		x = 200
		y = 0
		counter = 0
		self.currentLevel = 0
		
		self.levelList = []
		self.scoreList = []
		self.buttons = []
		self.readLevels()
		self.backButton = gui.Button(position=(150, 550), size=(500, 25), parent=self.app, text="Back to Title Screen")
		self.backButton.onClick = self.handleBack

			
		self.wait = True
		
	def readLevels(self):
		"""
			Open the manifest, figure out what the levels are.
			Create buttons for each level.
		"""
		f = open("data/levels/manifest.mf")
		lineNumber = 0
		x = 125
		y = 25
		for line in f:
			if lineNumber != 0:
				#create a button for the level
				#show best time and whether or not treasure has been found
				#only enable the button if the player has unlocked it
				filename = line.strip()
				levelFile = open("data/levels/" + filename)
				buttonName = levelFile.readline().strip()
				score = int(levelFile.readline().strip())
				levelFile.readline()
				levelFile.readline()
				levelFile.readline()
				found = (levelFile.readline().strip().split(":")[4] == "True")
				self.levelDict[buttonName] = filename
				self.levelList.append(buttonName)
				self.scoreList.append(score)
				nameString = buttonName + "  (Best Time:  "
				nameString += str(score) + "ms, Treasure "
				if found != True:
					nameString += "not "
				nameString += "found.)"
				button = gui.Button(position = (x, y), size=(550, 25), parent=self.app, text=nameString)
				button.onClick = self.handleStart
				self.buttons.append(button)
				if lineNumber - 1 > self.currentLevel:
					button.enabled = False
				levelFile.close()
				y += 25
			elif lineNumber == 0:
				#first line contains what level the player is on
				self.currentLevel = int(line.strip())
			lineNumber += 1
		f.close()
		
	def nextLevel(self):
		"""
			Unlock the next level.
		"""
		#about to start the next game level
		self.controller.getScene(2).reset()
		self.levelMarker += 1
		if self.levelMarker >= len(self.levels):
			self.levelMarker = 0
		#self.startButton.text = self.levels[self.levelMarker]
		self.wait = True
		
		
	def restartLevel(self):
		"""
			Reset the current level.
		"""
		self.controller.getScene(2).reset()
		self.wait = True

	def handleEvent(self, event):
		"""
			Handles any events generated by the user.
		"""
		#quit when the user clicks the "X"
		if event.type == pygame.QUIT:
			sys.exit()
			
	def tick(self, time):
		"""
			Update the gui.
		"""
		self.app.update()
		
	def draw(self, surface):
		"""
			Draw the Adrift load gui.
		"""
		self.app.draw()
			
	def handleStart(self, button):
		"""
			Start the game.
		"""
		self.controller.getScene(2).reset()
		#figure out what button was pressed, load the corresponding level
		#change to the game screen
		for i in xrange(len(self.levelList)):
			if button.text.startswith(self.levelList[i]):
				self.controller.getScene(2).loadLevel(self.levelDict[self.levelList[i]])
		self.controller.changeScene(2)
		
	def handleBack(self, button):
		"""
			Go back to the title screen.
		"""
		startScreen = self.controller.getScene(0)
		startScreen.drawArrow = False
		startScreen.drawArrowTimer = 0
		startScreen.waveSound = pygame.mixer.Sound(load_sound("ocean.ogg"))
		startScreen.waveSound.play(loops=-1)
		self.controller.changeScene(0)
		
	def advance(self, levelName, score, foundDoubloon):
		"""
			Go through the list of levels.
			Update the new high score if applicable.
			If we haven't beaten this level before, advance it one tick.
		"""
		for i in xrange(len(self.levelList)):
			if levelName == self.levelList[i]:
				#update score
				if self.scoreList[i] > score or foundDoubloon:
					if self.scoreList[i] > score:
						self.scoreList[i] = score
					word = ''
					if not foundDoubloon:
						word = 'not '
					self.buttons[i].text = levelName + "  (Best Time:  " + str(self.scoreList[i]) + "ms, Treasure " + word + "found.)"
					#write the new score to file
					oldFile = open("data/levels/" + self.levelDict[levelName])
					oldLines = oldFile.readlines()
					oldFile.close()
					newFile = open("data/levels/" + self.levelDict[levelName], "w")
					for j in xrange(len(oldLines)):
						if j == 1:
							newFile.write(str(self.scoreList[i]) + "\n")
						elif j == 5:
							parts = oldLines[j].strip().split(":")
							doubloonLine = parts[0] + ':' + parts[1] + ':' + parts[2] + ':' + parts[3] + ':' + str(foundDoubloon) + '\n'
							newFile.write(doubloonLine)
						else:
							newFile.write(oldLines[j])
					newFile.close()
				#unlock a level if necessary
				if i + 1 > self.currentLevel and i + 1 != len(self.levelList):
					self.currentLevel = self.currentLevel + 1
					self.buttons[i + 1].enabled = True
					#write the new manifest to file
					oldFile = open("data/levels/manifest.mf")
					oldLines = oldFile.readlines()
					oldFile.close()
					newFile = open("data/levels/manifest.mf", "w")
					for j in xrange(len(oldLines)):
						if j != 0:
							newFile.write(oldLines[j])
						else:
							newFile.write(str(self.currentLevel) + "\n")
					newFile.close()
					
