#client.Bullet.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

from Boat import Boat

class Bullet(AtlanSprite):
	"""
		Created by something with a canon.
		Travel with an x and y velocity.
	"""

	def __init__(self, myBounds, m, vX0, vY0):
		"""
			Create a new Bullet with a rectangle bounds, a reference to AdriftGameScreen, and an x/y velocity.
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.master = m
		self.images = getImages("bullet")
		self.image = self.images[0]
		self.animation = None
		self.dead = False
		self.velocityX = vX0
		self.velocityY = vY0
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def checkCollision(self, others):
		"""
			Collision detection handled in Boat class.
		"""
		pass
					
					
	def draw(self, surface):
		"""
			Draw to the screen in local coordinates.
		"""
		surface.blit(self.image, pygame.Rect(self.camX, self.camY, self.bounds.width, self.bounds.height))
			
	def update(self, delta):
		"""
			Update position based on velocity.
			If this object leaves the world's boundry * 2, then kill it.
		"""
		self.bounds.centerx += self.velocityX
		self.bounds.centery += self.velocityY
		if self.bounds.centerx >= (self.master.worldWidth * 2) or self.bounds.centerx < (-1 * self.master.worldWidth):
			self.dead = True
		if self.bounds.centery >= (self.master.worldHeight * 2) or self.bounds.centery < (-1 * self.master.worldHeight):
			self.dead = True
			
	def setWind(self, magnitude, direction):
		"""
			Bullets are not affected by wind.
		"""
		pass
		
	def isBoat(self):
		"""
			Not a boat.
		"""
		return False