#client.Canon.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

from Boat import Boat

from Bullet import Bullet

class Canon(AtlanSprite):
	"""
		Obstacle that can fire a bullet at the Ship if it get close.
	"""

	def __init__(self, myBounds, m):
		"""
			Create the Canon sprite.
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.master = m
		self.images = getImages("canon")
		self.image = self.images[0]
		self.background = getImages("block")[0]
		self.original = self.image.copy()
		self.imageBounds = self.bounds
		self.animation = None
		self.dead = False
		self.canFire = True
		self.fireCounter = 0
		#200 pixel track range
		self.trackRange = 200
		self.angle = 0
		#fire bullets at speed of 5
		self.firingSpeed = 5
		self.backgroundCamX = 0
		self.backgroundCamY = 0
		self.canonCamX = 0
		self.canonCamY = 0
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def calculateCameraCoords(self, rect):
		"""
			Set camera coordinates.
		"""
		self.backgroundCamX = self.bounds.x - rect.x
		self.backgroundCamY = self.bounds.y - rect.y
		self.canonCamX = self.imageBounds.x - rect.x
		self.canonCamY = self.imageBounds.y - rect.y
		
	def checkCollision(self, others):
		"""
			Boat class handles collision detection.
		"""
		pass
					
	def draw(self, surface):
		"""
			Draw the canon to the screen in screen coords.
		"""
		surface.blit(self.background, pygame.Rect(self.backgroundCamX, self.backgroundCamY, self.bounds.width, self.bounds.height))
		surface.blit(self.image, pygame.Rect(self.canonCamX, self.canonCamY, self.imageBounds.width, self.imageBounds.height))
		
	def update(self, delta):
		"""
			Update logic.
			Can only shoot once per 2 seconds.
			Only shoot at close by targets.
			The angle of canon image should match the firing angle.
		"""
		if not self.canFire:
			self.fireCounter += delta
			if self.fireCounter >= 2000:
				self.fireCounter = 0
				self.canFire = True
		if distance(self.bounds.centerx, self.bounds.centery, self.master.boat.bounds.centerx, self.master.boat.bounds.centery) <= self.trackRange:
			self.align(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
			if self.canFire:
				self.fire(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
				self.canFire = False
				
	def align(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety.
			Rotate in that direction
		"""
		inRads = math.atan2(self.bounds.y - targetY, self.bounds.x - targetX)
		degrees = inRads * 57.295
		self.rotate(180 - degrees)
		
	def rotate(self, degrees):
		"""
			Rotate our image.
			Degrees = from x axis.
			We want it to be from the y axis.
		"""
		self.angle = degrees
		self.image = pygame.transform.rotate(self.original, degrees)
		oldCenter = self.bounds.center
		self.imageBounds = self.image.get_rect()
		self.imageBounds.center = oldCenter
				
	def fire(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety.
			Create a new bullet in that direction with a set V0.
			Add the bullet to master's list of sprites.
			Play an explosion noise.
		"""
		bulletBounds = pygame.Rect(self.bounds.x, self.bounds.y, 5, 5)
		bulletBounds.centerx = self.bounds.centerx
		bulletBounds.centery = self.bounds.centery
		vX = self.firingSpeed * (math.cos(self.angle / 57.295))
		vY = self.firingSpeed * (math.sin(self.angle / 57.295))
		#lol screen coordinates
		vY = vY * -1
		vX = vX
		g = Bullet(bulletBounds, self.master, vX, vY)
		self.master.sprites.add(g)
		s = pygame.mixer.Sound(load_sound("canon.ogg"))
		s.play()
			
	def setWind(self, magnitude, direction):
		"""
			Canons are unaffected by wind.
		"""
		pass
		
	def isBoat(self):
		"""
			Not a boat.
		"""
		return False