#Cloud.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

import random



class Cloud(AtlanSprite):
	"""
		Clouds are groups of semi-transparent white pixels centered around an x,y coordinate.
		Exactly where the specs are is random, but all specs are within the rectangle.
		Search www.reinebold.com for my procedural cloud algorithm.
	"""

	def __init__(self, myBounds):
		"""
			Generate a bunch of cloud specs around a certain x,y coordinate.
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.bounds.width = random.randint(50, 100)
		self.bounds.height = random.randint(50, 100)
		self.flowCounter = 0
		self.windStrength = 0
		self.windDirection = 0
		self.proSurface = pygame.Surface((self.bounds.width, self.bounds.height), flags=pygame.SRCALPHA).convert_alpha()
		#self.proSurface.set_colorkey((255, 0, 0))
		self.proSurface.lock()
		center = (self.bounds.x + (self.bounds.width / 2), self.bounds.y + (self.bounds.height / 2))
		for i in xrange((self.bounds.width * self.bounds.height) / 200):
			#center = placed randomly within bounds (30% to 70% section)
			centerX = random.randint(int(self.bounds.width * 0.3), int(self.bounds.width * 0.7))
			centerY = random.randint(int(self.bounds.height * 0.3), int(self.bounds.height * 0.7))
			radius = random.randint(3, 10)
			pygame.draw.circle(self.proSurface, (255, 255, 255, 128), (centerX, centerY), radius)
		self.proSurface.unlock()
		self.trueX = self.bounds.x / 1.0
		self.trueY = self.bounds.y / 1.0
		self.SCROLL = 1
		self.WRAP = 2
		self.mode = self.WRAP
		
	def draw(self, surface):
		"""
			Draw the cloud to the surface.
		"""
		if self.mode == self.SCROLL:
			surface.blit(self.proSurface, pygame.Rect(self.camX, self.camY, self.bounds.width, self.bounds.height))
		else:
			surface.blit(self.proSurface, self.bounds)
		
	def update(self, delta):
		"""
			Called every frame to update animation timing.  Advances the animation by time "delta."
		"""
		self.flowCounter += delta
		if self.flowCounter > 0:
			self.flowCounter = 0
			self.flow(self.windStrength, self.windDirection)
			
		#If in the start screen, the clouds will wrap around the local screen.  If not in start screen mode, the clouds wrap around once they exit the world.
		if self.mode == self.WRAP:	
			if (self.bounds.x + self.bounds.width) > 800 + self.bounds.width:
				self.trueX = -1 * self.bounds.width
			if (self.bounds.y + self.bounds.height) > 600 + self.bounds.height:
				self.trueY = -1 * self.bounds.height
			if self.bounds.y < -1 * self.bounds.height:
				self.trueY = 600
			if self.bounds.x < -1 * self.bounds.width:
				self.trueX = 800
		else:
			if (self.bounds.x + self.bounds.width) > 1800 + self.bounds.width:
				self.trueX = -1 * (self.bounds.width + 200)
			if (self.bounds.y + self.bounds.height) > 1800 + self.bounds.height:
				self.trueY = -1 * (self.bounds.height + 200)
			if self.bounds.y < -1 * (self.bounds.height + 200):
				self.trueY = 1800
			if self.bounds.x < -1 * (self.bounds.width + 200):
				self.trueX = 1800
			
	def setWind(self, magnitude, direction):
		"""
			Set the wind's magnitude and direction for this cloud.
		"""
		self.windStrength = magnitude
		self.windDirection = direction
		
	def flow(self, speed, angle):
		"""
			Move the cloud by the given speed and angle of the wind.
			Clouds move twice as fast as the player boat.
		"""
		radians = float(angle) / 57.295
		xDisp = math.cos(radians) * speed
		yDisp = math.sin(radians) * speed
		self.trueX += xDisp
		self.trueY -= yDisp
		self.bounds.x = int(self.trueX)
		self.bounds.y = int(self.trueY)
		
	def calculateDistance(self, point1, point2):
		"""
			Calculate the absolute value of the distance between point 1 and point 2.
		"""
		return math.sqrt(((point1[0] - point2[0]) ** 2) +  ((point1[1] - point2[1]) ** 2))