#Fireworks.py

import math

import random 

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

class Fireworks(AtlanSprite):
	"""
		Firework special effect when the player beats a level.
	"""

	def __init__(self, m):
		"""
			Create a list to hold the explosions.
		"""
		AtlanSprite.__init__(self)
		self.bounds = pygame.Rect(0, 0, 800, 600)
		self.master = m
		self.animation = None
		self.explosions = []
		self.counter = 0
					
	def draw(self, surface):
		"""
			Draw all of the fireworks explosions to the surface.
		"""
		for explosion in self.explosions:
			explosion.draw(surface)
		
	def update(self, time):
		"""
			Add a new explosion to the list of explosions every tick.
			Update each of the explosions.
			If an explosion is finished, get rid of it.
		"""
		self.counter = self.counter + 1
		if self.counter % 1 == 0:
			self.explosions.append(Explosion(self.master))
		for explosion in self.explosions:
			explosion.update(time)
			if explosion.isDone():
				self.explosions.remove(explosion)
		
#I usually don't like more than one class per file,
#but it seemed appropriate in this case.		
		
class Explosion(AtlanSprite):
	"""
		Individual firework visual effect.
	"""


	def __init__(self, m):
		"""
			Pick a random color to draw in.
			A Firework is a bunch of particles that zip out from
			a randomly picked center location every 30 degrees.
		"""
		AtlanSprite.__init__(self)
		self.bounds = pygame.Rect(0, 0, 800, 600)
		self.master = m
		self.animation = None
		self.color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
		self.center = (random.randint(0, self.master.worldWidth), random.randint(0, self.master.worldHeight))
		self.locations = []
		self.per = 30
		for i in xrange(0, 360, self.per):
			self.locations.append([self.center[0], self.center[1]])
		self.finished = False
		self.counter = 0
		self.velocity = 10
					
	def draw(self, surface):
		"""
			Draw each particle of the Firework.
		"""
		for location in self.locations:
			pygame.draw.circle(surface, self.color, location, 1)
		
	def update(self, time):
		"""
			Move the particles through time.
			They travel away from the center of the explosion.
			When the 50 ticks have elapsed, the firework is done.
		"""
		self.counter = self.counter + 1
		if self.counter % 2 == 0:
			for i in xrange(len(self.locations)):
				angle = self.per * i
				self.locations[i][0] += int((math.cos(angle / 57.295)) * self.velocity)
				self.locations[i][1] += int((math.sin(angle / 57.295)) * self.velocity)
		if self.counter % 50 == 0:
			self.finished = True
		
	def isDone(self):
		"""
			Return whether or not the Firework has finished exploding.
			This is after 50 ticks.
		"""
		return self.finished
