#GunPatrolBoat.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

from Bullet import Bullet

class GunPatrolBoat(AtlanSprite):
	"""
		Obstacle that can move either left-right or up-down.
		Has a canon that can rotate and fire at the player.
	"""

	def __init__(self, myBounds, m, mySpeed, distance, hor):
		"""
			Create the GunPatrolBoat.
			myBounds = rectangle location
			m = reference to AdriftGameScreen
			mySpeed = speed to travel
			distance = how many pixels to patrol before turning around
			hor = boolean for is going horizontal
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.horizontal = None
		self.master = m
		self.images = getImages("pirate")
		self.image = self.images[0]
		self.canonImage = getImages("canon")[0]
		self.originalCanon = self.canonImage.copy()
		self.canonBounds = Rect(self.bounds.x, self.bounds.y, 25, 25)
		self.canonBounds.centerx = self.bounds.centerx
		self.canonBounds.centery = self.bounds.centery
		self.fireCounter = 0
		self.trackRange = 200
		self.angle = 0
		self.firingSpeed = 5
		self.canFire = True
		self.gunAngle = self.angle
		self.animation = None
		#flip image if not going horizontal
		if not hor:
			self.image = pygame.transform.rotate(self.image, -90)
			self.horizontal = False
		else:
			self.horizontal = True
		#flip image if going left
		if mySpeed < 0:
			self.image = pygame.transform.rotate(self.image, 180)
			self.movePositive = False
		else:
			self.movePositive = True
		self.distanceTraveled = 0
		self.maxDistance = distance
		self.speed = abs(mySpeed)
		self.dead = False
		self.shipCamX = 0
		self.shipCamY = 0
		self.canonCamX = 0
		self.canonCamY = 0
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def calculateCameraCoords(self, rect):
		"""
			Calculate relative screen coordinates.
		"""
		self.shipCamX = self.bounds.x - rect.x
		self.shipCamY = self.bounds.y - rect.y
		self.canonCamX = self.canonBounds.x - rect.x
		self.canonCamY = self.canonBounds.y - rect.y
		
		
	def checkCollision(self, others):
		"""
			Boat class handles collision detection
		"""
		pass
					
	def draw(self, surface):
		"""
			Draw to the screen.
		"""
		surface.blit(self.image, pygame.Rect(self.shipCamX, self.shipCamY, self.bounds.width, self.bounds.height))
		surface.blit(self.canonImage, pygame.Rect(self.canonCamX, self.canonCamY, self.canonBounds.width, self.canonBounds.height))
			
	def setWind(self, magnitude, direction):
		"""
			Ignore wind.
		"""
		pass
		
	def update(self, delta):
		"""
			Update internal logic by one tick.
			Move the ship according to velocity.
			Turn around if necessary.
			Handle canon logic.
		"""
		#move
		if self.horizontal:
			if self.movePositive:
				self.bounds.x += self.speed
			else:
				self.bounds.x -= self.speed
		else:
			if self.movePositive:
				self.bounds.y += self.speed
			else:
				self.bounds.y -= self.speed
		self.canonBounds.center = self.bounds.center
		self.distanceTraveled += self.speed
		#turn around if necessary
		if self.distanceTraveled >= self.maxDistance:
			self.distanceTraveled = 0
			self.movePositive = not self.movePositive
			self.image = pygame.transform.rotate(self.image, 180)
			self.hitmask = pygame.surfarray.array_alpha(self.image)
			
		#check if can fire canon
		if not self.canFire:
			self.fireCounter += delta
			if self.fireCounter >= 2000:
				self.fireCounter = 0
				self.canFire = True	
		#fire canon if in range
		if distance(self.bounds.centerx, self.bounds.centery, self.master.boat.bounds.centerx, self.master.boat.bounds.centery) <= self.trackRange:
			self.align(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
			if self.canFire:
				self.fire(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
				self.canFire = False
	
	def rotateCanon(self, degrees):
		"""
			Degrees = from x axis.
			We want it to be from the y axis.
		"""
		self.canonAngle = degrees
		self.canonImage = pygame.transform.rotate(self.originalCanon, degrees)
		oldCenter = self.bounds.center
		self.canonBounds = self.canonImage.get_rect()
		self.canonBounds.center = oldCenter
				
	def align(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety.
			Rotate in that direction.
		"""
		inRads = math.atan2(self.bounds.y - targetY, self.bounds.x - targetX)
		degrees = inRads * 57.295
		self.rotateCanon(180 - degrees)
				
	def fire(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety.
			Create a new bullet in that direction with a set V0.
			Add the bullet to master's list of sprites.
		"""
		bulletBounds = pygame.Rect(self.bounds.x, self.bounds.y, 5, 5)
		bulletBounds.centerx = self.bounds.centerx
		bulletBounds.centery = self.bounds.centery
		vX = self.firingSpeed * (math.cos(self.canonAngle / 57.295))
		vY = self.firingSpeed * (math.sin(self.canonAngle / 57.295))
		#lol screen coordinates
		vY = vY * -1
		vX = vX
		g = Bullet(bulletBounds, self.master, vX, vY)
		self.master.sprites.add(g)
		#gunshot noise
		s = pygame.mixer.Sound(load_sound("canon.ogg"))
		s.play()
		
	def isBoat(self):
		"""
			Not a boat.
		"""
		return False