#client.GunPirate
#Arrrrr...now with guns!

from pygame import Rect

from common.Utility import *

import random

import math

from engine.AtlanSprite import AtlanSprite

from Bullet import Bullet


class GunPirate(AtlanSprite):
	"""
		A boat that moves on the wind like the player does.
		Moves at half speed of player.
		Also has a canon on board which can fire at player.
	"""

	def __init__(self, myBounds, m):
		"""
			Create the GunPirate sprite.
		"""
		AtlanSprite.__init__(self)
		self.animation = None
		self.master = m
		self.bounds = myBounds
		self.images = getImages("pirate")
		self.canonImage = getImages("canon")[0]
		self.image = self.images[0]
		self.original = self.image.copy()
		self.originalCanon = self.canonImage.copy()
		self.clock = 0
		self.flowCounter = 0
		self.windStrength = 0
		self.windDirection = 0
		self.trueX = self.bounds.x / 1.0
		self.trueY = self.bounds.y / 1.0
		self.imageBounds = self.bounds
		self.dead = False
		self.canonBounds = Rect(self.bounds.x, self.bounds.y, 25, 25)
		self.canonBounds.centerx = self.bounds.centerx
		self.canonBounds.centery = self.bounds.centery
		self.fireCounter = 0
		self.trackRange = 200
		self.angle = 0
		self.firingSpeed = 5
		self.canFire = True
		self.gunAngle = self.angle
		self.shipCamX = 0
		self.shipCamY = 0
		self.canonCamX = 0
		self.canonCamY = 0
		#see pixel perfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def calculateCameraCoords(self, rect):
		"""
			Calculate relative screen coordinates.
		"""
		self.shipCamX = self.imageBounds.x - rect.x
		self.shipCamY = self.imageBounds.y - rect.y
		self.canonCamX = self.canonBounds.x - rect.x
		self.canonCamY = self.canonBounds.y - rect.y
		
	def flow(self, speed, angle):
		"""
			Move according to the speed and angle of the wind.
			Does not check for collisions:
				None of the other obstacles can hit it (same velocity).
				If the player touches, level ends.
		"""
		radians = float(angle) / 57.295
		xDisp = math.cos(radians) * speed * 0.5
		yDisp = math.sin(radians) * speed * 0.5
		self.trueX += xDisp
		self.trueY -= yDisp
		self.bounds.x = int(self.trueX)
		self.bounds.y = int(self.trueY)
		
		
		
	def draw(self, surface):
		"""
			Draw to the game screen.
		"""
		surface.blit(self.image, pygame.Rect(self.shipCamX, self.shipCamY, self.imageBounds.width, self.imageBounds.height))
		surface.blit(self.canonImage, pygame.Rect(self.canonCamX, self.canonCamY, self.canonBounds.width, self.canonBounds.height))
			
	def update(self, delta):
		"""
			Called every frame to update animation timing.  Advances the animation by time "delta."
		"""
		#move with the wind
		self.flowCounter += delta
		if self.flowCounter > 0:
			self.flowCounter = 0
			self.flow(self.windStrength, self.windDirection)
			self.rotateShip(self.windDirection)
		#check if canon can fire
		if not self.canFire:
			self.fireCounter += delta
			if self.fireCounter >= 2000:
				self.fireCounter = 0
				self.canFire = True
				
		#check if in range
		if distance(self.bounds.centerx, self.bounds.centery, self.master.boat.bounds.centerx, self.master.boat.bounds.centery) <= self.trackRange:
			self.align(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
			if self.canFire:
				self.fire(self.master.boat.bounds.centerx, self.master.boat.bounds.centery)
				self.canFire = False
				
	def rotateCanon(self, degrees):
		"""
			Degrees = from x axis.
			We want it to be from the y axis.
		"""
		self.canonAngle = degrees
		self.canonImage = pygame.transform.rotate(self.originalCanon, degrees)
		oldCenter = self.bounds.center
		self.canonBounds = self.canonImage.get_rect()
		self.canonBounds.center = oldCenter
		
	def rotateShip(self, degrees):
		"""
			Rotate ship image according to angle of wind.
		"""
		self.angle = degrees
		self.image = pygame.transform.rotate(self.original, degrees)
		oldCenter = self.bounds.center
		self.bounds = self.image.get_rect()
		self.imageBounds = self.image.get_rect()
		self.imageBounds.center = oldCenter
		self.bounds.center = oldCenter
		self.bounds.center = oldCenter
		self.rotateCanon(degrees)
		#we updated the image, have to update hitmask
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def setWind(self, magnitude, direction):
		"""
			Set strength and angle of wind.
		"""
		self.windStrength = magnitude
		self.windDirection = direction
		
	def checkCollision(self, others):
		"""
			Boat class does collision detection.
		"""
		pass
		
	def align(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety
			Rotate in that direction.
		"""
		inRads = math.atan2(self.bounds.y - targetY, self.bounds.x - targetX)
		degrees = inRads * 57.295
		self.rotateCanon(180 - degrees)
				
	def fire(self, targetX, targetY):
		"""
			Figure out the angle between our x axis and the targetx, targety.
			Create a new bullet in that direction with a set V0.
			Add the bullet to master's list of sprites.
		"""
		bulletBounds = pygame.Rect(self.bounds.x, self.bounds.y, 5, 5)
		bulletBounds.centerx = self.bounds.centerx
		bulletBounds.centery = self.bounds.centery
		vX = self.firingSpeed * (math.cos(self.canonAngle / 57.295))
		vY = self.firingSpeed * (math.sin(self.canonAngle / 57.295))
		#lol screen coordinates
		vY = vY * -1
		vX = vX
		g = Bullet(bulletBounds, self.master, vX, vY)
		self.master.sprites.add(g)
		s = pygame.mixer.Sound(load_sound("canon.ogg"))
		s.play()
		
		
	def isBoat(self):
		"""
			Not a boat.
		"""	
		return False