#PatrolBoat.py

import math

import pygame

from pygame import Rect

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

class PatrolBoat(AtlanSprite):
	"""
		Obstacle that moves back and forth.
	"""

	def __init__(self, myBounds, m, mySpeed, distance, hor):
		"""
			Create a new PatrolBoat sprite.
			myBounds = rectangle bounds
			m = reference to AdriftGameScreen
			mySpeed = how fast to move
			distance = how far in pixels until you turn around
			hor = is the movement horizontal
		"""
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.horizontal = None
		self.master = m
		self.images = getImages("pirate")
		self.image = self.images[0]
		self.animation = None
		#rotate image to fit vertical movement
		if not hor:
			self.image = pygame.transform.rotate(self.image, -90)
			self.horizontal = False
		else:
			self.horizontal = True
		#rotate image to fit leftward movement
		if mySpeed < 0:
			self.image = pygame.transform.rotate(self.image, 180)
			self.movePositive = False
		else:
			self.movePositive = True
		self.distanceTraveled = 0
		self.maxDistance = distance
		self.speed = abs(mySpeed)
		self.dead = False
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
		
	def checkCollision(self, others):
		"""
			Handle collision detection mostly in Boat class.
		"""
		pass
					
	def draw(self, surface):
		"""
			Draw to the screen in local coordinates.
		"""
		surface.blit(self.image, pygame.Rect(self.camX, self.camY, self.bounds.width, self.bounds.height))
			
	def setWind(self, magnitude, direction):
		"""
			PatrolBoats are not affected by wind.
		"""
		pass
		
	def update(self, delta):
		"""
			Update logic one tick.
		"""
		#move
		if self.horizontal:
			if self.movePositive:
				self.bounds.x += self.speed
			else:
				self.bounds.x -= self.speed
		else:
			if self.movePositive:
				self.bounds.y += self.speed
			else:
				self.bounds.y -= self.speed
		self.distanceTraveled += self.speed
		#reverse direction if necessary
		if self.distanceTraveled >= self.maxDistance:
			self.distanceTraveled = 0
			self.movePositive = not self.movePositive
			#flip image
			self.image = pygame.transform.rotate(self.image, 180)
			#adjust hit mask, see PixelPerfect
			self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def isBoat(self):
		"""
			Not a boat.
		"""
		return False