#client.Pirate
#Arrrrr

from pygame import Rect

from common.Utility import *

import random

import math

from engine.AtlanSprite import AtlanSprite

class Pirate(AtlanSprite):
	"""
		Obstacle that moves along with the wind.
		Travels at half the speed as objects of the Boat class.
	"""

	def __init__(self, myBounds, m):
		"""
			Create a new Pirate obstacle.
		"""
		AtlanSprite.__init__(self)
		self.animation = None
		self.master = m
		self.bounds = myBounds
		self.images = getImages("pirate")
		self.image = self.images[0]
		self.original = self.image.copy()
		self.clock = 0
		self.flowCounter = 0
		self.windStrength = 0
		self.windDirection = 0
		self.trueX = self.bounds.x / 1.0
		self.trueY = self.bounds.y / 1.0
		self.imageBounds = self.bounds
		self.dead = False
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def flow(self, speed, angle):
		"""
			Move position based on speed and angle of wind
		"""
		radians = float(angle) / 57.295
		xDisp = math.cos(radians) * speed * 0.5
		yDisp = math.sin(radians) * speed * 0.5
		self.trueX += xDisp
		self.trueY -= yDisp
		self.bounds.x = int(self.trueX)
		self.bounds.y = int(self.trueY)
		
	def calculateCameraCoords(self, rect):
		"""
			Calculate local screen coords.
		"""
		self.camX = self.imageBounds.x - rect.x
		self.camY = self.imageBounds.y - rect.y
		
	def draw(self, surface):
		"""
			Draw to the screen.
		"""
		surface.blit(self.image, pygame.Rect(self.camX, self.camY, self.imageBounds.width, self.imageBounds.height))
			
	def update(self, delta):
		"""
			Called every frame to update animation timing.  Advances the animation by time "delta."
		"""
		self.flowCounter += delta
		if self.flowCounter > 0:
			self.flowCounter = 0
			self.flow(self.windStrength, self.windDirection)
			self.rotate(self.windDirection)
				
	def rotate(self, degrees):
		"""
			Rotate image based on wind angle.
		"""
		self.image = pygame.transform.rotate(self.original, degrees)
		oldCenter = self.bounds.center
		self.bounds = self.image.get_rect();
		self.imageBounds = self.image.get_rect()
		self.bounds.center = oldCenter
		self.imageBounds.center = oldCenter
		self.bounds.center = oldCenter
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def setWind(self, magnitude, direction):
		"""
			Set the wind strength and angle for internal use.
		"""
		self.windStrength = magnitude
		self.windDirection = direction
		
	def checkCollision(self, others):
		"""
			Handle collision detection in Boat class.
		"""
		pass
					
	def isBoat(self):
		"""
			Not a boat.
		"""
		return False