#common.Utility.py
"""
	Contains constants and functions for image loading.
"""

import pygame
import os
import math

version = "Adrift 1.0"

loadedImages = {}
fonts = {}

def distance(x1, y1, x2, y2):
	"""
		Return the geometric distance between two points.
	"""
	return math.sqrt(math.pow((x1 - x2), 2) + math.pow((y1 - y2), 2))

def floatIsZero(f):
	"""
		Return if a float is equal to zero.
		Avoids nasty computer floating point errors.
	"""
	tolerance = 0.005
	if f + tolerance > 0.0 and f - tolerance < 0.0:
		return True
	else:
		return False

def getGameFont():
	"""
		Returns the game font.  If the game font has not been loaded before, loads it from disk.
		Otherwise, load it from memory.
		current game font == freesansbold.ttf, font size of 12
	"""
	if not fonts.has_key("freesansbold.ttf"):
		fonts["freesansbold.ttf"] = pygame.font.Font("freesansbold.ttf", 12)
	return fonts["freesansbold.ttf"]


def smartLoadImage(fileName):
	"""
		Returns a pygame image from a filename.  If the image has not been loaded before, it loads it from disk.
		Otherwise, load it from memory.
	"""
	if not loadedImages.has_key(fileName):
		loadedImages[fileName] = load_image(fileName)
	return loadedImages[fileName]

def getImages(descrip):
	"""
	Given a string image description (not a filename), returns the list of images associated with it.
	This funciton is to be used with Netsprites.
	"""
	images = []
	if descrip == "boat":
		images.append(smartLoadImage("sailboat.png"))
	elif descrip == "block":
		images.append(smartLoadImage("block.png"))
	elif descrip == "canon":
		images.append(smartLoadImage("turret.png"))
	elif descrip == "bullet":
		images.append(smartLoadImage("bullet.png"))
	elif descrip == "pirate":
		images.append(smartLoadImage("evilsailboat.png"))
	elif descrip == "doubloon":
		images.append(smartLoadImage("doubloon.png"))
	elif descrip == "seagull":
		images.append(smartLoadImage("seagull1.png"))
		images.append(smartLoadImage("seagull2.png"))
		images.append(smartLoadImage("seagull3.png"))
	elif descrip == "goal":
		images.append(smartLoadImage("goal.png"))
	return images
	
def getImage(descrip):
	"""
	Given a string image description (not a filename), returns one image associated with it.
	This function is to be used with background or other static images.
	Does not load the image from memory, because these are large images and would take up a lot of space.
	"""
	if descrip == "oceanBackground":
		return load_image("oceanBackground.png")
	elif descrip == "title":
		return load_image("adriftTitle.png")
	elif descrip == "okSign":
		return load_image("okSign.png")
	
def load_image(name):
	"""
	Given a filename (not a path), finds the file in the data/images directory and loads and returns a pygame
	image object found at that file.
	If the specified file cannot be found, generate an error to close the program.
	Thanks to the Pyman example on www.pygame.org for how to do this.
	"""
	fullname = os.path.join('data', 'images')
	fullname = os.path.join(fullname, name)
	try:
		image = pygame.image.load(fullname)
	except pygame.error, message:
		print('Cannot load image: ' + fullname)
		raise SystemExit, message
	image = image.convert_alpha()
	return image
	
def load_sound(name):
	"""
		Given a filename (not a path), finds the path to the file.
	"""
	fullname = os.path.join('data', 'sound')
	fullname = os.path.join(fullname, name)
	return fullname